@@ -11,11 +11,6 @@
 #include <algorithm>
 #include <cstring>
 #include <map>
-#include <sstream>
-
-#include <boost/lexical_cast.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
 
 #include <OpenThreads/ReentrantMutex>
 #include <OpenThreads/ScopedLock>
@@ -30,9 +25,11 @@
 #include <osgDB/ReadFile>
 #include <osgDB/Registry>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/scene/util/SGReaderWriterOptions.hxx>
+#include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/scene/util/SGSceneFeatures.hxx>
 #include <simgear/scene/util/SplicingVisitor.hxx>
 #include <simgear/structure/SGExpression.hxx>
@@ -131,20 +128,20 @@ Effect* makeEffect(const string& name,
     string absFileName
         = SGModelLib::findDataFile(effectFileName, options);
     if (absFileName.empty()) {
-        SG_LOG(SG_INPUT, SG_ALERT, "can't find \"" << effectFileName << "\"");
-        return 0;
+        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::LoadEffectsShaders, "Couldn't find Effect:" + effectFileName);
+        return nullptr;
     }
     SGPropertyNode_ptr effectProps = new SGPropertyNode();
     try {
         readProperties(absFileName, effectProps.ptr(), 0, true);
     }
     catch (sg_io_exception& e) {
-        SG_LOG(SG_INPUT, SG_ALERT, "error reading \"" << absFileName << "\": "
-               << e.getFormattedMessage());
-        return 0;
+        simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::LoadEffectsShaders, e.getFormattedMessage(),
+                               e.getLocation());
+        return nullptr;
     }
     ref_ptr<Effect> result = makeEffect(effectProps.ptr(), realizeTechniques,
-                                        options);
+                                        options, SGPath::fromUtf8(absFileName));
     if (result.valid()) {
         OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(effectMutex);
         pair<EffectMap::iterator, bool> irslt
@@ -161,28 +158,32 @@ Effect* makeEffect(const string& name,
 
 Effect* makeEffect(SGPropertyNode* prop,
                    bool realizeTechniques,
-                   const SGReaderWriterOptions* options)
+                   const SGReaderWriterOptions* options,
+                   const SGPath& filePath)
 {
     // Give default names to techniques and passes
     vector<SGPropertyNode_ptr> techniques = prop->getChildren("technique");
     for (int i = 0; i < (int)techniques.size(); ++i) {
         SGPropertyNode* tniqProp = techniques[i].ptr();
-        if (!tniqProp->hasChild("name"))
+        if (!tniqProp->hasChild("name")) {
             setValue(tniqProp->getChild("name", 0, true),
-                     boost::lexical_cast<string>(i));
+                     std::to_string(i));
+        }
         vector<SGPropertyNode_ptr> passes = tniqProp->getChildren("pass");
         for (int j = 0; j < (int)passes.size(); ++j) {
             SGPropertyNode* passProp = passes[j].ptr();
-            if (!passProp->hasChild("name"))
+            if (!passProp->hasChild("name")) {
                 setValue(passProp->getChild("name", 0, true),
-                         boost::lexical_cast<string>(j));
+                         std::to_string(j));
+            }
             vector<SGPropertyNode_ptr> texUnits
                 = passProp->getChildren("texture-unit");
             for (int k = 0; k < (int)texUnits.size(); ++k) {
                 SGPropertyNode* texUnitProp = texUnits[k].ptr();
-                if (!texUnitProp->hasChild("name"))
+                if (!texUnitProp->hasChild("name")) {
                     setValue(texUnitProp->getChild("name", 0, true),
-                             boost::lexical_cast<string>(k));
+                             std::to_string(k));
+                }
             }
         }
     }
@@ -219,6 +220,7 @@ Effect* makeEffect(SGPropertyNode* prop,
             if (!effect.valid()) {
                 effect = new Effect;
                 effect->setName(nameProp->getStringValue());
+                effect->setFilePath(filePath.isNull() ? parent->filePath() : filePath);
                 effect->root = new SGPropertyNode;
                 mergePropertyTrees(effect->root, prop, parent->root);
                 effect->parametersProp = effect->root->getChild("parameters");
@@ -236,12 +238,13 @@ Effect* makeEffect(SGPropertyNode* prop,
                 effect->generator = parent->generator;  // Copy the generators
             }
         } else {
-            SG_LOG(SG_INPUT, SG_ALERT, "can't find base effect " <<
-                   inheritProp->getStringValue());
-            return 0;
+            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::LoadEffectsShaders,
+                                   string{"couldn't find base effect to inherit from:"} + inheritProp->getStringValue(), filePath);
+            return nullptr;
         }
     } else {
         effect = new Effect;
+        effect->setFilePath(filePath);
         effect->setName(nameProp->getStringValue());
         effect->root = prop;
         effect->parametersProp = effect->root->getChild("parameters");
@@ -268,9 +271,9 @@ Effect* makeEffect(SGPropertyNode* prop,
             effect->realizeTechniques(options);
         }
         catch (BuilderException& e) {
-            SG_LOG(SG_INPUT, SG_ALERT, "Error building technique: "
-                   << e.getFormattedMessage());
-            return 0;
+            simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::LoadEffectsShaders,
+                                   "Failed to build technique:" + e.getFormattedMessage(), filePath);
+            return nullptr;
         }
     }
     return effect.release();
