@@ -1,4 +1,4 @@
-// materialmgr.cxx -- class to handle material properties
+// matlib.cxx -- class to handle material properties
 //
 // Written by Curtis Olson, started May 1998.
 //
@@ -31,18 +31,17 @@
 
 #include <string.h>
 #include <string>
+#include <mutex>
 
 #include <osgDB/Registry>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/props/condition.hxx>
 #include <simgear/scene/tgdb/userdata.hxx>
-#include <simgear/threads/SGThread.hxx>
-#include <simgear/threads/SGGuard.hxx>
 
 #include "mat.hxx"
 
@@ -56,7 +55,7 @@ using std::string;
 class SGMaterialLib::MatLibPrivate
 {
 public:
-    SGMutex mutex;
+    std::mutex mutex;
 };
 
 // Constructor
@@ -66,7 +65,7 @@ SGMaterialLib::SGMaterialLib ( void ) :
 }
 
 // Load a library of material properties
-bool SGMaterialLib::load( const string &fg_root, const string& mpath,
+bool SGMaterialLib::load( const SGPath &fg_root, const SGPath& mpath,
         SGPropertyNode *prop_root )
 {
     SGPropertyNode materialblocks;
@@ -82,7 +81,7 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath,
     osg::ref_ptr<osgDB::Options> options
         = new osgDB::Options;
     options->setObjectCacheHint(osgDB::Options::CACHE_ALL);
-    options->setDatabasePath(fg_root);
+    options->setDatabasePath(fg_root.utf8Str());
 
     simgear::PropertyList blocks = materialblocks.getChildren("region");
     simgear::PropertyList::const_iterator block_iter = blocks.begin();
@@ -129,12 +128,13 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath,
 
 		// Now build all the materials for this set of areas and conditions
 
+		const std::string region = node->getStringValue("name");
 		const simgear::PropertyList materials = node->getChildren("material");
 		simgear::PropertyList::const_iterator materials_iter = materials.begin();
 		for (; materials_iter != materials.end(); materials_iter++) {
 			const SGPropertyNode *node = materials_iter->get();
 			SGSharedPtr<SGMaterial> m =
-					new SGMaterial(options.get(), node, prop_root, arealist, condition);
+					new SGMaterial(options.get(), node, prop_root, arealist, condition, region);
 
 			std::vector<SGPropertyNode_ptr>names = node->getChildren("name");
 			for ( unsigned int j = 0; j < names.size(); j++ ) {
