@@ -25,8 +25,7 @@
 #include <cstring>
 #include <cassert>
 
-#include <boost/bind.hpp>
-
+#include <osg/Version>
 #include <osg/Geode>
 #include <osg/MatrixTransform>
 #include <osgDB/ReadFile>
@@ -36,12 +35,13 @@
 #include <osgDB/FileNameUtils>
 
 #include <simgear/compiler.h>
-#include <simgear/structure/exception.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/props/condition.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/props/condition.hxx>
 #include <simgear/scene/util/SGNodeMasks.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
+#include <simgear/structure/exception.hxx>
 
 #include "modellib.hxx"
 #include "SGReaderWriterXML.hxx"
@@ -49,6 +49,7 @@
 #include "animation.hxx"
 #include "particles.hxx"
 #include "model.hxx"
+#include "SGLight.hxx"
 #include "SGText.hxx"
 #include "SGMaterialAnimation.hxx"
 
@@ -56,7 +57,7 @@ using namespace std;
 using namespace simgear;
 using namespace osg;
 
-static osg::Node *
+static std::tuple<int, osg::Node *>
 sgLoad3DModel_internal(const SGPath& path,
                        const osgDB::Options* options,
                        SGPropertyNode *overlay = 0);
@@ -81,6 +82,7 @@ SGReaderWriterXML::readNode(const std::string& name,
                             const osgDB::Options* options) const
 {
     std::string fileName = osgDB::findDataFile(name, options);
+    simgear::ErrorReportContext ec{"model-xml", fileName};
 
     osg::Node *result=0;
     try {
@@ -88,10 +90,12 @@ SGReaderWriterXML::readNode(const std::string& name,
         if (!p.exists()) {
           return ReadResult::FILE_NOT_FOUND;
         }
-        
-        result=sgLoad3DModel_internal(p, options);
+
+        static std::tuple<int, osg::Node *> retval;
+        int num_anims;
+        std::tie(num_anims, result) = sgLoad3DModel_internal(p, options);
     } catch (const sg_exception &t) {
-        SG_LOG(SG_INPUT, SG_ALERT, "Failed to load model: " << t.getFormattedMessage()
+        SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load model: " << t.getFormattedMessage()
           << "\n\tfrom:" << fileName);
         result=new osg::Node;
     }
@@ -160,13 +164,13 @@ void makeEffectAnimations(PropertyList& animation_nodes,
         SGPropertyNode* typeProp = animProp->getChild("type");
         if (!typeProp)
             continue;
-        
+
         const char* typeString = typeProp->getStringValue();
         if (!strcmp(typeString, "material")) {
             effectProp
                 = SGMaterialAnimation::makeEffectProperties(animProp);
         } else if (!strcmp(typeString, "shader")) {
-            
+
             SGPropertyNode* shaderProp = animProp->getChild("shader");
             if (!shaderProp || strcmp(shaderProp->getStringValue(), "chrome"))
                 continue;
@@ -193,10 +197,10 @@ void makeEffectAnimations(PropertyList& animation_nodes,
 
         }
     }
-    animation_nodes.erase(remove_if(animation_nodes.begin(),
-                                    animation_nodes.end(),
-                                    !boost::bind(&SGPropertyNode_ptr::valid,
-                                                 _1)),
+    animation_nodes.erase(std::remove_if(animation_nodes.begin(),
+                                         animation_nodes.end(),
+                                         [] (const SGPropertyNode_ptr& ptr) {
+                                             return !ptr.valid(); }),
                           animation_nodes.end());
 }
 }
@@ -215,7 +219,7 @@ private:
     osg::Node::NodeMask nodeMaskSet;
     osg::Node::NodeMask nodeMaskClear;
 };
-    
+
 }
 
 namespace {
@@ -229,7 +233,7 @@ namespace {
             return (typeString == "pick") || (typeString == "knob") || (typeString == "slider") || (typeString == "touch");
         }
     };
-    
+
     bool removeNamedNode(osg::Group* aGroup, const std::string& aName)
     {
         int nKids = aGroup->getNumChildren();
@@ -240,28 +244,30 @@ namespace {
                 return true;
             }
         }
-        
+
         for (int i = 0; i < nKids; i++) {
             osg::Group* childGroup = aGroup->getChild(i)->asGroup();
             if (!childGroup)
                 continue;
-            
+
             if (removeNamedNode(childGroup, aName))
                 return true;
         } // of child groups traversal
-        
+
         return false;
     }
 }
 
-static osg::Node *
+static std::tuple<int, osg::Node *>
 sgLoad3DModel_internal(const SGPath& path,
                        const osgDB::Options* dbOptions,
                        SGPropertyNode *overlay)
 {
     if (!path.exists()) {
-      SG_LOG(SG_INPUT, SG_ALERT, "Failed to load file: \"" << path << "\"");
-      return NULL;
+        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
+                               "Failed to load model XML: not found", path);
+        SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << path << "\"");
+        return std::make_tuple(0, (osg::Node*)NULL);
     }
 
     osg::ref_ptr<SGReaderWriterOptions> options;
@@ -270,25 +276,28 @@ sgLoad3DModel_internal(const SGPath& path,
     SGPath modelpath(path);
     SGPath texturepath(path);
     SGPath modelDir(modelpath.dir());
-    
+    int animationcount = 0;
+
     SGSharedPtr<SGPropertyNode> prop_root = options->getPropertyNode();
     if (!prop_root.valid())
         prop_root = new SGPropertyNode;
     // The model data appear to be only used in the topmost model
     osg::ref_ptr<SGModelData> data = options->getModelData();
     options->setModelData(0);
-    
+
     osg::ref_ptr<osg::Node> model;
     osg::ref_ptr<osg::Group> group;
     SGPropertyNode_ptr props = new SGPropertyNode;
     bool previewMode = (dbOptions->getPluginStringData("SimGear::PREVIEW") == "ON");
-    
+
     // Check for an XML wrapper
     if (modelpath.extension() == "xml") {
        try {
             readProperties(modelpath, props);
         } catch (const sg_exception &t) {
-            SG_LOG(SG_INPUT, SG_ALERT, "Failed to load xml: "
+            simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::XMLModelLoad,
+                                   "Failed to load model XML:" + t.getFormattedMessage(), t.getLocation());
+            SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load xml: "
                    << t.getFormattedMessage());
             throw;
         }
@@ -296,24 +305,30 @@ sgLoad3DModel_internal(const SGPath& path,
             copyProperties(overlay, props);
 
         if (previewMode && props->hasChild("nopreview")) {
-            return NULL;
+            return std::make_tuple(0, (osg::Node *) NULL);
         }
-        
+
         if (props->hasValue("/path")) {
             string modelPathStr = props->getStringValue("/path");
             modelpath = SGModelLib::findDataFile(modelPathStr, NULL, modelDir);
-            if (modelpath.isNull())
+            if (modelpath.isNull()) {
+                simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::ThreeDModelLoad,
+                                       "Model not found:" + modelPathStr, path);
                 throw sg_io_exception("Model file not found: '" + modelPathStr + "'",
-                        path);
+                                      path, {}, false);
+            }
 
             if (props->hasValue("/texture-path")) {
                 string texturePathStr = props->getStringValue("/texture-path");
                 if (!texturePathStr.empty())
                 {
                     texturepath = SGModelLib::findDataFile(texturePathStr, NULL, modelDir);
-                    if (texturepath.isNull())
+                    if (texturepath.isNull()) {
+                        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::LoadingTexture,
+                                               "Texture file not found:" + texturePathStr, path);
                         throw sg_io_exception("Texture file not found: '" + texturePathStr + "'",
                                 path);
+                    }
                 }
             }
         } else {
@@ -333,12 +348,20 @@ sgLoad3DModel_internal(const SGPath& path,
         if (!texturepath.extension().empty())
             texturepath = texturepath.dir();
 
-        options->setDatabasePath(texturepath.local8BitStr());
+        options->setDatabasePath(texturepath.utf8Str());
         osgDB::ReaderWriter::ReadResult modelResult;
-        modelResult = osgDB::readNodeFile(modelpath.local8BitStr(), options.get());
-        if (!modelResult.validNode())
+#if OSG_VERSION_LESS_THAN(3,4,1)
+        modelResult = osgDB::readNodeFile(modelpath.utf8Str(), options.get());
+#else
+        modelResult = osgDB::readRefNodeFile(modelpath.utf8Str(), options.get());
+#endif
+        if (!modelResult.validNode()) {
+            simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::XMLModelLoad,
+                                   "Failed to load 3D model:" + modelResult.message(), modelpath);
             throw sg_io_exception("Failed to load 3D model:" + modelResult.message(),
-                                  modelpath);
+                                  modelpath, {}, false);
+        }
+
         model = copyModel(modelResult.getNode());
         // Add an extra reference to the model stored in the database.
         // That is to avoid expiring the object from the cache even if
@@ -399,13 +422,16 @@ sgLoad3DModel_internal(const SGPath& path,
 
         SGPath submodelpath;
         osg::ref_ptr<osg::Node> submodel;
-        
+
         string subPathStr = sub_props->getStringValue("path");
-        SGPath submodelPath = SGModelLib::findDataFile(subPathStr, 
+        SGPath submodelPath = SGModelLib::findDataFile(subPathStr,
           NULL, modelDir);
 
         if (submodelPath.isNull()) {
-          SG_LOG(SG_INPUT, SG_ALERT, "Failed to load file: \"" << subPathStr << "\"");
+          SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << subPathStr << "\"");
+          simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
+                                 "Couldn't find file for submodel:" + subPathStr,
+                                 SGPath::fromUtf8(subPathStr));
           continue;
         }
 
@@ -418,15 +444,19 @@ sgLoad3DModel_internal(const SGPath& path,
             }
         }
 
+        simgear::ErrorReportContext("submodel", submodelPath.utf8Str());
+
         try {
-            submodel = sgLoad3DModel_internal(submodelPath, options.get(),
+            int num_anims;
+            std::tie(num_anims, submodel) = sgLoad3DModel_internal(submodelPath, options.get(),
                                               sub_props->getNode("overlay"));
+            animationcount += num_anims;
         } catch (const sg_exception &t) {
-            SG_LOG(SG_INPUT, SG_ALERT, "Failed to load submodel: " << t.getFormattedMessage()
+            SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load submodel: " << t.getFormattedMessage()
               << "\n\tfrom:" << t.getOrigin());
             continue;
         }
-        
+
         if (!submodel)
             continue;
 
@@ -473,7 +503,7 @@ sgLoad3DModel_internal(const SGPath& path,
         // Load panels
         vector<SGPropertyNode_ptr> panel_nodes = props->getChildren("panel");
         for (unsigned i = 0; i < panel_nodes.size(); i++) {
-            SG_LOG(SG_INPUT, SG_DEBUG, "Loading a panel");
+            SG_LOG(SG_IO, SG_DEBUG, "Loading a panel");
             osg::ref_ptr<osg::Node> panel = load_panel(panel_nodes[i]);
             if (panel_nodes[i]->hasValue("name"))
                 panel->setName(panel_nodes[i]->getStringValue("name"));
@@ -481,7 +511,8 @@ sgLoad3DModel_internal(const SGPath& path,
         }
     }
 
-    if (dbOptions->getPluginStringData("SimGear::PARTICLESYSTEM") != "OFF") {
+    auto particlesManager = ParticlesGlobalManager::instance();
+    if (particlesManager->isEnabled()) { //dbOptions->getPluginStringData("SimGear::PARTICLESYSTEM") != "OFF") {
         std::vector<SGPropertyNode_ptr> particle_nodes;
         particle_nodes = props->getChildren("particlesystem");
         for (unsigned i = 0; i < particle_nodes.size(); ++i) {
@@ -491,12 +522,12 @@ sgLoad3DModel_internal(const SGPath& path,
             if (i==0) {
                 if (!texturepath.extension().empty())
                     texturepath = texturepath.dir();
-                
-                options2->setDatabasePath(texturepath.local8BitStr());
+
+                options2->setDatabasePath(texturepath.utf8Str());
             }
-            group->addChild(Particles::appendParticles(particle_nodes[i],
-                                                       prop_root,
-                                                       options2.get()));
+            group->addChild(particlesManager->appendParticles(particle_nodes[i],
+                                                              prop_root,
+                                                              options2.get()));
         }
     }
 
@@ -508,15 +539,23 @@ sgLoad3DModel_internal(const SGPath& path,
                         options.get()));
     }
 
+    std::vector<SGPropertyNode_ptr> light_nodes;
+    light_nodes = props->getChildren("light");
+    for (unsigned i = 0; i < light_nodes.size(); ++i) {
+        group->addChild(SGLight::appendLight(light_nodes[i],
+                                             prop_root,
+                                             options.get()));
+    }
+
     PropertyList effect_nodes = props->getChildren("effect");
     PropertyList animation_nodes = props->getChildren("animation");
-    
+
     if (previewMode) {
         PropertyList::iterator it;
         it = std::remove_if(animation_nodes.begin(), animation_nodes.end(), ExcludeInPreview());
         animation_nodes.erase(it, animation_nodes.end());
     }
-    
+
     // Some material animations (eventually all) are actually effects.
     makeEffectAnimations(animation_nodes, effect_nodes);
     {
@@ -525,7 +564,7 @@ sgLoad3DModel_internal(const SGPath& path,
         group = static_cast<Group*>(modelWithEffects.get());
     }
 
-    simgear::SGTransientModelData modelData(group.get(), prop_root, options.get(), path.local8BitStr());
+    simgear::SGTransientModelData modelData(group.get(), prop_root, options.get(), path.utf8Str());
 
     for (unsigned i = 0; i < animation_nodes.size(); ++i) {
         if (previewMode && animation_nodes[i]->hasChild("nopreview")) {
@@ -535,21 +574,32 @@ sgLoad3DModel_internal(const SGPath& path,
             } // of object-names in the animation
             continue;
         }
-        /*
-         * Setup the model data for the node currently being animated.
-         */
-        modelData.LoadAnimationValuesForElement(animation_nodes[i], i);
+        
+        try {
+            /*
+             * Setup the model data for the node currently being animated.
+             */
+            modelData.LoadAnimationValuesForElement(animation_nodes[i], i);
 
-        /// OSGFIXME: duh, why not only model?????
-        SGAnimation::animate(modelData);
+            /// OSGFIXME: duh, why not only model?????
+            SGAnimation::animate(modelData);
+        } catch (sg_exception& e) {
+            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
+                                   "Couldn't load animation " + animation_nodes[i]->getNameString()
+                                    + ":" + e.getFormattedMessage(),
+                                   modelpath);
+            throw;
+        }
     }
-    
+
+    animationcount += animation_nodes.size();
+
     if (!needTransform && group->getNumChildren() < 2) {
         model = group->getChild(0);
         group->removeChild(model.get());
         if (data.valid())
             data->modelLoaded(modelpath.utf8Str(), props, model.get());
-        return model.release();
+        return std::make_tuple(animationcount, model.release());
     }
     if (data.valid())
         data->modelLoaded(modelpath.utf8Str(), props, group.get());
@@ -559,6 +609,7 @@ sgLoad3DModel_internal(const SGPath& path,
         osgDB::writeNodeFile(*group, outputfile);
     }
 
-    return group.release();
-}
+    SG_LOG(SG_GENERAL, SG_DEBUG, "Model " << path << " animation count: " << animationcount);
 
+    return std::make_tuple(animationcount, group.release());
+}
